/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GUILOADER_GUILOADER_H
#define GUILOADER_GUILOADER_H

#include <gtk/gtk.h>

#define GUILOADER_VERSION (2*1000000 + 7*1000 + 1)

#ifdef _WIN32
	#ifdef GUILOADER_LIBRARY
		#define GUILOADER_API __declspec(dllexport)
	#else
		#define GUILOADER_API __declspec(dllimport)
	#endif
#else
	#define GUILOADER_API __attribute__ ((visibility("default")))
#endif

#define GUIXML_DECL              \
  {                              \
    const char data[]=

#define GUIXML_LOAD(loader)      \
    ;                            \
    gui_loader_load_from_string(loader, data, sizeof(data)-1); \
  }

#define GUIXML_(SSS)

G_BEGIN_DECLS

#define GUI_TYPE_LOADER (gui_loader_get_type())
#define GUI_LOADER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), GUI_TYPE_LOADER, GuiLoader))
#define GUI_LOADER_CLASS(meta) (G_TYPE_CHECK_CLASS_CAST((meta), GUI_TYPE_LOADER, GuiLoaderClass))
#define GUI_IS_LOADER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), GUI_TYPE_LOADER))
#define GUI_IS_LOADER_CLASS(meta) (G_TYPE_CHECK_CLASS_TYPE((meta), GUI_TYPE_LOADER))
#define GUI_LOADER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), GUI_TYPE_LOADER, GuiLoaderClass))

typedef struct _GuiLoader GuiLoader;
typedef struct _GuiLoaderClass GuiLoaderClass;
typedef struct _GuiLoaderInit GuiLoaderInit;
typedef struct _GuiConnector GuiConnector;
typedef struct _GuiConnectInfo GuiConnectInfo;

struct _GuiLoader {
	GObject parent;
};

struct _GuiLoaderClass {
	GObjectClass parent;
	GObjectClass * parent_class;
	GuiLoaderInit * init;
};

struct _GuiConnectInfo {
	GObject * object;
	const gchar * signame;
	gboolean after;
};

GUILOADER_API gint guiloader_version(void);

GUILOADER_API void guiloader_enter_check(const gchar * msg);
GUILOADER_API void guiloader_leave(void);

GUILOADER_API GType gui_loader_get_type(void);
GUILOADER_API GuiLoader * gui_loader_new(void);
GUILOADER_API void gui_loader_set_preview_mode(GuiLoader * loader, gboolean preview);
GUILOADER_API void gui_loader_set_steal_objects(GuiLoader * loader, gboolean steal);
GUILOADER_API void gui_loader_load_from_string(GuiLoader * loader, const gchar * data, gssize size);
GUILOADER_API gboolean gui_loader_load_from_file(GuiLoader * loader, const gchar * filename, GError ** error);
GUILOADER_API void gui_loader_clear(GuiLoader * loader);
GUILOADER_API GQueue * gui_loader_query_connectors(GuiLoader * loader, const gchar * name, gint count);
GUILOADER_API void gui_loader_get_connect_info(GuiLoader * loader, const GuiConnector * connector, GuiConnectInfo * info);
GUILOADER_API void gui_loader_connect_signals(GuiLoader * loader, const gchar * name, GCallback func, gpointer userdata, gint count);
GUILOADER_API gpointer gui_loader_bind_object(GuiLoader * loader, const gchar * name);

#define guiloader_enter() guiloader_enter_check(NULL)
#define gui_loader_connect_signal(loader, name, func, userdata) gui_loader_connect_signals(loader, name, func, userdata, 1)

G_END_DECLS

#endif
