/*
 *  GuiLoader examples
 *  Copyright (c) 2006 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef _WIN32
	#include <gdk/gdkwin32.h>
#endif

#include <glib/gi18n.h>
#include <guiloader.h> 

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

typedef struct _Application Application;

struct _Application {
	GuiLoader * loader;
	GtkWidget * window;
};

gboolean OnDeleteEvent(GtkWidget * widget, GdkEvent * event, Application * app) { 
	gtk_main_quit();
	return TRUE; 
}

void ApplicationInit(Application * app) {
	app->loader= gui_loader_new();

	GUIXML_DECL
		#include "hello.gui.h"
	GUIXML_LOAD(app->loader)

	app->window= gui_loader_bind_object(app->loader, "window1");
	gui_loader_connect_signal(app->loader, "onDeleteEvent", G_CALLBACK(OnDeleteEvent), app);
}
	
void ApplicationDone(Application * app) { 
	g_object_unref(app->loader); 
}

int Main(int argc, char ** argv) {
	Application app;

	gtk_init(&argc, &argv);
	guiloader_enter();

	#if ENABLE_NLS
		bindtextdomain(PACKAGE, LOCALEDIR);
		bind_textdomain_codeset(PACKAGE, "UTF-8");
		textdomain(PACKAGE);
	#endif

	ApplicationInit(&app);
	gtk_widget_show(app.window);
	gtk_main();
	ApplicationDone(&app);

	guiloader_leave();		
	return 0;
}

#ifdef _WIN32
	int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
		{ return Main(__argc, __argv); }
#else
	int main(int argc, char ** argv) 
		{ return Main(argc, argv); }
#endif
